import {
	ChangeDetectionStrategy,
	Component,
	forwardRef,
	Input,
	OnChanges,
	OnInit,
	SimpleChanges
} from '@angular/core';
import {
	ControlValueAccessor,
	FormBuilder,
	FormControl,
	FormGroup,
	NG_VALUE_ACCESSOR
} from '@angular/forms';

@Component({
	selector: 'gp-input',
	templateUrl: './gp-input.component.html',
	styleUrls: ['./gp-input.component.scss'],
	providers: [
		{
			provide: NG_VALUE_ACCESSOR,
			useExisting: forwardRef(() => GpInputComponent),
			multi: true
		}
	],
	changeDetection: ChangeDetectionStrategy.OnPush
})
export class GpInputComponent implements ControlValueAccessor, OnInit, OnChanges {

	// Название
	@Input() public header = '';

	// Текст ошибки или булево
	@Input() public error: string | boolean = false;

	// Дизейбл инпута
	@Input() public disable = false;

	// Тип: text или password
	@Input() public inputType = 'text';

	// Placeholder инпута
	@Input() public placeholder = '';

	// Дефолтный класс формы
	public defaultClass = 'gp-input__form';

	// Список классов формы
	public formClass = this.defaultClass;

	// Конструктор формы
	public formBuilder: FormBuilder = new FormBuilder();

	// Содаем реактивную форму для дальнейшей подписки на изменения данных
	public formGroup: FormGroup = this.formBuilder.group({
		input: new FormControl()
	});

	// Храним в переменной доступ к инпуту
	public readonly input = this.formGroup.get('input') as FormControl;

	public ngOnInit(): void {
		// Подписка на события инпута
		this.subscribeToInputEvents();
	}

	// Слушаем изменения входящих данных
	public ngOnChanges(changes: SimpleChanges): void {
		if (this.input) {
			if (changes.error && changes.error.currentValue) {
				this.formClass = this.setClass('error');
			} else if (changes.disable && changes.disable.currentValue) {
				this.formClass = this.setClass('disabled');
				this.input.disable();
			} else if (this.isError) {
				this.formClass = this.setClass('error');
				this.input.enable();
			} else if (this.disable) {
				this.formClass = this.setClass('disable');
				this.input.enable();
			} else {
				this.formClass = this.setClass();
				this.input.enable();
			}
		}

	}

	// Подписываемся на изменения событий и уведомляем родителя
	private subscribeToInputEvents(): void {
		if (this.input) {
			this.input
				.valueChanges
				.subscribe((data: string) => {
					this.propagateChange(data);
				});
		}
	}

	// Проверка на наличие ошибки
	private get isError(): boolean {
		return this.error === '' || this.error === false;
	}

	// Проверяем на состояние Ошибки и добавляем класс
	public setFocus(): void {
		if (!this.isError) {
			this.formClass = this.setClass('focus');
		}
	}

	// Проверяем на состояние Ошибки и сбрасываем классы
	public removeFocus(): void {
		if (!this.isError) {
			this.formClass = this.defaultClass;
		}
	}

	// Задаем список классов для формы
	public setClass(name?: string): string {
		return name === 'string'
			? `${this.defaultClass} ${this.defaultClass}--${name}`
			: this.defaultClass;
	}

	// Записываем данные переданные от родителя
	public writeValue(value: string): void {
		if (this.input) {
			this.input.setValue(value);
		}
	}

	// Необходимые методы для работы с ngModel
	public registerOnChange(fn: any): void {
		this.propagateChange = fn;
	}

	// tslint:disable-next-line:no-empty
	public registerOnTouched(fn: any): void {}

	// tslint:disable-next-line:no-empty
	public propagateChange = (fn: any): void => {};
}
